%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Real data analysis code for GPR            %%%%%
%%%%% The code is for backcross or DH population %%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%% Reference: Cui, Y.H., D.-Y. Kim, and J. Zhu. (2006) 
%%%%% On the generalized Poisson regression mixture model for mapping quantitative trait loci with count data. 
%%%%% Genetics 174(4): 21592172. 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%warning off;
global y n;  

%### there are 12 chromosome ###
%### 1=heterozygote  0=homozygote
interval=2; %%% the increment to scan the genome
%***************************************************
%      get the phynotype value and marker infor    *
%***************************************************
yy=load('pheno.txt'); %%% the phenotype data file. Missing values are denoted as -1.
MM1=load('marker.txt'); %%% the marker data file. Missing values are denoted as 88.
dd=load('distance.txt'); %%% file contains the genetic distance. It is edited as a matrix with columns denoting the number of chromosomes
mp=load('posi.txt'); %%% file contains the number of markers for each chromosome. It is a vector.

zo=find(yy==-1); %%% find the missing phenotype value
yy(zo)=[]; %%% delete missing values
MM1(zo,:)=[]; %%% delete missing values for corresponding markers

%%% read the chromosome map
CHR1=dd(1:mp(1),1); CHR2=dd(1:mp(2),2); CHR3=dd(1:mp(3),3);
CHR4=dd(1:mp(4),4); CHR5=dd(1:mp(5),5); CHR6=dd(1:mp(6),6);
CHR7=dd(1:mp(7),7); CHR8=dd(1:mp(8),8); CHR9=dd(1:mp(9),9); 
CHR10=dd(1:mp(10),10); CHR11=dd(1:mp(11),11); CHR12=dd(1:mp(12),12);
%%% get the scan position for each chromosom by a 2cM increment
d1=sort(unique([CHR1(2:mp(1))',(0:(max(CHR1)/interval))*interval]));          n1 =length(d1);
d2=sort(unique([CHR2(2:mp(2))',(0:(max(CHR2)/interval))*interval])); 	      n2 =length(d2);
d3=sort(unique([CHR3(2:mp(3))',(0:(max(CHR3)/interval))*interval]));          n3 =length(d3);
d4=sort(unique([CHR4(2:mp(4))',(0:(max(CHR4)/interval))*interval]));          n4 =length(d4);
d5=sort(unique([CHR5(2:mp(5))',(0:(max(CHR5)/interval))*interval]));          n5 =length(d5);
d6=sort(unique([CHR6(2:mp(6))',(0:(max(CHR6)/interval))*interval]));          n6 =length(d6);
d7=sort(unique([CHR7(2:mp(7))',(0:(max(CHR7)/interval))*interval]));          n7 =length(d7);
d8=sort(unique([CHR8(2:mp(8))',(0:(max(CHR8)/interval))*interval]));          n8 =length(d8);
d9=sort(unique([CHR9(2:mp(9))',(0:(max(CHR9)/interval))*interval]));          n9 =length(d9);
d10=sort(unique([CHR10(2:mp(10))',(0:(max(CHR10)/interval))*interval]));      n10=length(d10);
d11=sort(unique([CHR11(2:mp(11))',(0:(max(CHR11)/interval))*interval]));      n11=length(d11);
d12=sort(unique([CHR12(2:mp(12))',(0:(max(CHR12)/interval))*interval]));      n12=length(d12);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
d=[d1 d2 d3 d4 d5 d6 d7 d8 d9 d10 d11 d12]; %%% list the total positions to scan
nn=sum([n1 n2 n3 n4 n5 n6 n7 n8 n9 n10 n11 n12]); %%% the total scan positions for all chromosomes
yy1=round(yy); %%% if yy is not integer, get the rounding integer value

%%%% Get the marker data %%%
M11=MM1(:,1:mp(1)); M12=MM1(:,(mp(1)+1):sum(mp(1:2)));M13=MM1(:,(sum(mp(1:2))+1):sum(mp(1:3)));M14=MM1(:,(sum(mp(1:3))+1):sum(mp(1:4)));
M15=MM1(:,(sum(mp(1:4))+1):sum(mp(1:5)));M16=MM1(:,(sum(mp(1:5))+1):sum(mp(1:6)));M17=MM1(:,(sum(mp(1:6))+1):sum(mp(1:7)));
M18=MM1(:,(sum(mp(1:7))+1):sum(mp(1:8)));M19=MM1(:,(sum(mp(1:8))+1):sum(mp(1:9)));M110=MM1(:,(sum(mp(1:9))+1):sum(mp(1:10)));
M111=MM1(:,(sum(mp(1:10))+1):sum(mp(1:11)));M112=MM1(:,(sum(mp(1:11))+1):sum(mp(1:12)));

LR=[];SE=[];
ppar=[]; posi=[];
InterID(1)=0;
for ww=1:2 % nn location to scan
        if ww<=n1 
            chrom1=CHR1;M_1=M11;posi1=1;
        elseif ww>n1 & ww<=sum([n1 n2])
            chrom1=CHR2;M_1=M12;posi1=2;
        elseif ww>sum([n1 n2]) & ww<=sum([n1 n2 n3])
            chrom1=CHR3;M_1=M13;posi1=3;
        elseif ww>sum([n1 n2 n3]) & ww<=sum([n1 n2 n3 n4])
            chrom1=CHR4;M_1=M14;posi1=4;
        elseif ww>sum([n1 n2 n3 n4]) & ww<=sum([n1 n2 n3 n4 n5])
            chrom1=CHR5;M_1=M15;posi1=5;
        elseif ww>sum([n1 n2 n3 n4 n5]) & ww<=sum([n1 n2 n3 n4 n5 n6])
            chrom1=CHR6;M_1=M16;posi1=6;
        elseif ww>sum([n1 n2 n3 n4 n5 n6]) & ww<=sum([n1 n2 n3 n4 n5 n6 n7])
            chrom1=CHR7;M_1=M17;posi1=7;
        elseif ww>sum([n1 n2 n3 n4 n5 n6 n7]) & ww<=sum([n1 n2 n3 n4 n5 n6 n7 n8])
            chrom1=CHR8;M_1=M18;posi1=8;
        elseif ww>sum([n1 n2 n3 n4 n5 n6 n7 n8]) & ww<=sum([n1 n2 n3 n4 n5 n6 n7 n8 n9])
            chrom1=CHR9;M_1=M19;posi1=9;
        elseif ww>sum([n1 n2 n3 n4 n5 n6 n7 n8 n9]) & ww<=sum([n1 n2 n3 n4 n5 n6 n7 n8 n9 n10])
            chrom1=CHR10;M_1=M110;posi1=10;
        elseif ww>sum([n1 n2 n3 n4 n5 n6 n7 n8 n9 n10]) & ww<=sum([n1 n2 n3 n4 n5 n6 n7 n8 n9 n10 n11])
            chrom1=CHR11;M_1=M111;posi1=11;
        elseif ww>sum([n1 n2 n3 n4 n5 n6 n7 n8 n9 n10 n11]) & ww<=sum([n1 n2 n3 n4 n5 n6 n7 n8 n9 n10 n11 n12])
            chrom1=CHR12;M_1=M112; posi1=12;
        end

        d_1=d(ww);
        loc1=length(find(chrom1(:)<=d_1));
        if loc1==length(chrom1)
            loc1=loc1-1;
        end
        % check if a scan is still in the same interval
        InterID(2)=loc1;
        % If enter a new interval, recalculate score vector and update all estimations           
        if InterID(1)~=InterID(2)
            y=yy1;
            mm=find(M_1(:,loc1)==88 | M_1(:,(loc1+1))==88); %%% find missing value for markers
            
            y(mm)=[]; %%% delete phenotypes with missing markers
            MMM1=[M_1(:,loc1) M_1(:,loc1+1)];
            MMM1(mm,:)=[]; %%% delete markers with missing values

            InterID(1)=InterID(2);
            n=length(y);
            
            %%% estimate parameters under the null %%%
            lm0=mean(y); ph0=0;
            L0=[1 2];
            beta02=[ph0; log(lm0)]; lll0=[]; beta01=beta02+1;
            while abs(L0(2)-L0(1))>1e-7 & abs(beta02(1)-beta01(1))>1e-7
               L0(1)=L0(2);
               beta01=beta02;
               
               Iph0=-lm0*(1+zeros(1,n))*y/(1+ph0*lm0)+(1+zeros(1,n))*(y.*(y-1)./(1+ph0*y))-lm0*(1+zeros(1,n))*(y-lm0)/(1+ph0*lm0)^2;
               Ilm0=(1+zeros(1,n))*(y-lm0)/(1+ph0*lm0)^2;
               u0=[Iph0; Ilm0];
               
               Iph02=lm0^2*(1+zeros(1,n))*y/(1+ph0*lm0)^2-(1+zeros(1,n))*(y.^2.*(y-1)./(1+ph0*y).^2)+(2*lm0^2*(1+zeros(1,n))*(y-lm0))/(1+ph0*lm0)^3;
               Iph0lm0=-(2*lm0*(1+zeros(1,n))*(y-lm0))/(1+ph0*lm0)^3;
               Ilm02=-(2*ph0*lm0*(1+zeros(1,n))*y+n*lm0*(1-ph0*lm0))/(1+ph0*lm0)^3;
               H0=[[Iph02 Iph0lm0];[Iph0lm0 Ilm02]];

               beta02=beta01-inv(H0)*u0;
               if 1+beta02(1)*max(y)<=0 | beta02(1)>5
                   beta02(1)=-1/(max(y)+1);
               elseif 1+beta02(1)*beta02(2)<=0
                  beta02(1)=-1/(lm0+1);
               end 
               ph0=beta02(1);
               lm0=exp(beta02(2));
               L0(2)=sum(log(Gpois2(y,ph0,lm0)));
               lll0=[lll0 L0(2)];
            end
        end
       
        s=fr((chrom1(loc1+1)-chrom1(loc1))/100);
        s1=fr((d_1-chrom1(loc1))/100);s2=fr((chrom1(loc1+1)-d_1)/100);
        p = fp(s1,s2,s);
    
        sphi=fn_m(MMM1(:,1),MMM1(:,2),p,n);  %%% prior dist

        %%% estimate parameters under the alternative
        L=[1 2];lll=[];
        beta2=[ph0; log(lm0); 0]; 
        while abs(L(2)-L(1))>1e-9
            L(1)=L(2);
            beta1=beta2;

            %%% E-step 
            cphi=[];            
            for j=1:n
                deno = sphi(j,:)*[pois(y(j),beta2(1),exp(beta2(2)+beta2(3))); pois(y(j),beta2(1),exp(beta2(2)))];
           
                cphi(j,1) = sphi(j,1)*pois(y(j),beta2(1),exp(beta2(2)+beta2(3)))/deno;
                cphi(j,2) = 1-cphi(j,1);
            end
            
            %%% M-step using Newton-Raphson iteration methods
            lm1=exp(beta2(2)+beta2(3)); lm2=exp(beta2(2));  
            ph=beta2(1);
                
            Iph=-lm1*cphi(:,1)'*y/(1+ph*lm1)+cphi(:,1)'*(y.*(y-1)./(1+ph*y))+(sum(cphi(:,1))*lm1^2-lm1*cphi(:,1)'*y)/(1+ph*lm1)^2+...
                -lm2*cphi(:,2)'*y/(1+ph*lm2)+cphi(:,2)'*(y.*(y-1)./(1+ph*y))+(sum(cphi(:,2))*lm2^2-lm2*cphi(:,2)'*y)/(1+ph*lm2)^2;
            Iu=cphi(:,1)'*(y-lm1)/(1+ph*lm1)^2+cphi(:,2)'*(y-lm2)/(1+ph*lm2)^2;
            Ia=cphi(:,1)'*(y-lm1)/(1+ph*lm1)^2;
            uu=[Iph; Iu; Ia];            

            Iph2=lm1^2*cphi(:,1)'*y/(1+ph*lm1)^2-cphi(:,1)'*(y.^2.*(y-1)./(1+ph*y).^2)+(2*lm1^2*cphi(:,1)'*(y-lm1))/(1+ph*lm1)^3+...
                 lm2^2*cphi(:,2)'*y/(1+ph*lm2)^2-cphi(:,2)'*(y.^2.*(y-1)./(1+ph*y).^2)+(2*lm2^2*cphi(:,2)'*(y-lm2))/(1+ph*lm2)^3;

            Iu2=-lm1*cphi(:,1)'*(2*ph*y-ph*lm1+1)/(1+ph*lm1)^3- lm2*cphi(:,2)'*(2*ph*y-ph*lm2+1)/(1+ph*lm2)^3;
            Ia2=-lm1*cphi(:,1)'*(2*ph*y-ph*lm1+1)/(1+ph*lm1)^3;
            
            Iphu=-2*lm1*cphi(:,1)'*(y-lm1)/(1+ph*lm1)^3- 2*lm2*cphi(:,2)'*(y-lm2)/(1+ph*lm2)^3;
            Ipha=-2*lm1*cphi(:,1)'*(y-lm1)/(1+ph*lm1)^3;
            Iua=-lm1*cphi(:,1)'*(1+ph*(2*y-lm1))/(1+ph*lm1)^3;
            
            Hiss=[[Iph2 Iphu Ipha];[Iphu Iu2 Iua];[Ipha Iua Ia2]];

            beta2=beta1-inv(Hiss)*uu;
            if 1+beta2(1)*max(y)<=0 
                beta2(1)=-1/(max(y)+1);
            %elseif 1+beta2(1)*beta2(2)<=0
            %    beta2(1)=-1/(lm0+1);
            end 
            ph=beta2(1);lm1=exp(beta2(2)+beta2(3));lm2=exp(beta2(2));  
            L(2)=sum(log(sphi(:,1).*Gpois2(y,ph,lm1)+sphi(:,2).*Gpois2(y,ph,lm2)));
            lll=[lll L(2)];
        end
        x=sqrt(-diag(inv(Hiss)));
        SE(ww,:)=x(1); %%% standard errors for parameters at each position
        %AIC(ww,:)=-2*L(2)+2*3; %%% the AIC information criteria
        %BIC(ww,:)=-2*L(2)+3*log(n);  %%% the BIC information criteria

        LR(ww,:)= -2*(L0(2)-L(2)); %%% LR vector contains the likelihood ratio test statistics at each test position
        ppar(ww,:)=beta2'; %%% ppar matrix contains all the parameter estimation at each test postions: the 1st col is phe, 2nd col is u and third col is a
        posi(ww,:)=posi1; %%% posi indicates chromosome positions
        [ww LR(ww,:) max(LR)]
end  %%% end the ww iteration
final=[posi d' LR ppar]; %%% final contains the chromosome position as well as the LR and parameter estimation

%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% code for permutation %%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% iter=200;
% for gg=1:iter
%     %%% permute the data
%     y=[];
%     vvv=randperm(length(yy1));
%     for kk=1:length(yy1)
%         y(kk,:)=yy1(vvv(kk),:);
%     end
%    %then use the above estimation code for permutation test 
% end